from airsim import MultirotorClient
from numpy.typing import NDArray
from envs import BaseEnvironment
import numpy as np

from tf_agents.specs.array_spec import BoundedArraySpec, ArraySpec
from tf_agents.typing.types import TimeStep, NestedArray
from tf_agents.trajectories import time_step as ts

class TestEnvironment(BaseEnvironment):
    def __init__(self, ip: str = None, client: MultirotorClient = None, handle_auto_reset: bool = False):

        self._action_spec = BoundedArraySpec(shape=(4,), dtype=np.float32, minimum=0, maximum=1, name='action')
        self._observation_spec = ArraySpec(shape=(6,), dtype=np.float64, name='observation')

        super(TestEnvironment, self).__init__(ip, client, handle_auto_reset)

    def action_spec(self) -> BoundedArraySpec:
        return self._action_spec
    
    def observation_spec(self) -> ArraySpec:
        return self._observation_spec
    
    def _reset(self) -> TimeStep:
        super()._reset()
        position = self._client.getMultirotorState().gps_location
        self._client.moveToGPSAsync(latitude=position.latitude, longitude=position.longitude, altitude=150, velocity=20).join()
        return ts.restart(self._get_observation())
    
    def _is_terminated(self, obs: NDArray) -> bool:
        return False
    
    def _is_truncated(self, obs: NDArray) -> bool:
        return False
    
    def _must_restart(self) -> bool:
        return False
    
    def _get_observation(self) -> NDArray:
        state = self._client.getMultirotorState()
        # geo_point = state.gps_location
        kinetics = state.kinematics_estimated
        acceleration = self._client.getImuData().linear_acceleration
        
        return np.array([acceleration.x_val, acceleration.y_val, acceleration.z_val])
    
    def _calculate_reward(self, obs: NDArray) -> np.float32:
        return np.float32(0)
