# import sys
# sys.path.append("../")

from envs import StartEnvironment
import tensorflow as tf
import airsim
import numpy as np

from tf_agents.environments.tf_py_environment import TFPyEnvironment
from tf_agents.agents.sac.sac_agent import SacAgent
from tf_agents.agents.ddpg.critic_network import CriticNetwork
from tf_agents.networks.actor_distribution_network import ActorDistributionNetwork
from tf_agents.agents.sac.tanh_normal_projection_network import TanhNormalProjectionNetwork
from tf_agents.train.utils import train_utils
from tf_agents.policies.py_tf_eager_policy import PyTFEagerPolicy
from tf_agents.policies.greedy_policy import GreedyPolicy

SIM_IP = "192.168.8.195"
actor_fc_layer_params = (10,)
critic_joint_fc_layer_params = (10,)
critic_learning_rate = 3e-4 # @param {type:"number"}
actor_learning_rate = 3e-4 # @param {type:"number"}
alpha_learning_rate = 3e-4 # @param {type:"number"}
target_update_tau = 0.005 # @param {type:"number"}
target_update_period = 1 # @param {type:"number"}
gamma = 0.99 # @param {type:"number"}
reward_scale_factor = 0.8 # @param {type:"number"}

tempdir="out/"

client = airsim.MultirotorClient(SIM_IP)

eval_policy = tf.saved_model.load("out/policy")

def _step_loop(env, steps: int, initial_time_step):
    time_step = initial_time_step
    time_steps = []
    for _ in range(steps):
        action = eval_policy.action(time_step)
        time_step = env.step(action)
        time_steps.append(time_step)

    return time_steps

def _execute_test(env, steps: int):
    time_step = env.reset()
    return _step_loop(env, steps, time_step)

def _make_env(desired_height: int) -> TFPyEnvironment:
    py_env = StartEnvironment(desired_height=desired_height, client=client)
    return TFPyEnvironment(py_env)

def _assert_last_time_steps(time_steps: []):
    last_steps = time_steps[-10:]
    last_observations = [ts.observation for ts in last_steps]
    last_observations = np.array(last_observations).flatten()

    for ts in last_steps:
        height_diffrence = ts.observation.numpy().flatten()[0]
        assert(abs(height_diffrence) < 1.0)

    assert(abs(last_observations.max() - last_observations.min()) < 1.0)

def test130():
    env = _make_env(130)
    time_steps = _execute_test(env, 1000)
    _assert_last_time_steps(time_steps)

def test150():
    env = _make_env(150)
    time_steps = _execute_test(env, 2000)
    _assert_last_time_steps(time_steps)

def test180():
    env = _make_env(180)
    time_steps = _execute_test(env, 4000)
    _assert_last_time_steps(time_steps)

def test150from180():
    env = _make_env(160)
    time_step = env.reset()

    position = client.getMultirotorState().gps_location
    client.moveToGPSAsync(latitude=position.latitude, longitude=position.longitude, altitude=180, velocity=10).join()

    time_steps =_step_loop(env, 2000, time_step)
    _assert_last_time_steps(time_steps)

def test_landing():
    env = _make_env(123.46241760253906)
    time_step = env.reset()

    position = client.getMultirotorState().gps_location
    client.moveToGPSAsync(latitude=position.latitude, longitude=position.longitude, altitude=140, velocity=10).join()

    time_steps =_step_loop(env, 2000, time_step)
    _assert_last_time_steps(time_steps)
