from envs import NavigationEnvironment
import train.utils as utils

from tf_agents.environments.tf_py_environment import TFPyEnvironment
from tf_agents.train.utils import train_utils
from tf_agents.policies.random_py_policy import RandomPyPolicy

SIM_IP = "192.168.8.195"

initial_train_episodes = 150
initial_train_steps_per_episode = 1000
initial_eval_steps = 2000

train_episodes = 10000
train_steps_per_episode = 4000
eval_steps = 6000
train_initial_steps = 0

replay_buffer_capacity = 100000

batch_size = 1024

critic_learning_rate = 1e-4
actor_learning_rate = 1e-4
alpha_learning_rate = 1e-4
gamma = 0.9

actor_fc_layer_params = (64,64)
critic_joint_fc_layer_params = actor_fc_layer_params

tempdir = "out/"
is_initial = utils.is_initial(tempdir)

utils.set_gpu()

train_env = NavigationEnvironment(ip=SIM_IP, dynamic_end=True, desired_latitude=47.641206516, desired_longitude=-122.130206082, initial_height=750, desired_height=750)
eval_env = NavigationEnvironment(ip=SIM_IP, dynamic_end=False, desired_latitude=47.641206516, desired_longitude=-122.130206082, initial_height=750, desired_height=750)
train_tf_env = TFPyEnvironment(train_env)
eval_tf_env = TFPyEnvironment(eval_env)

observation_spec = train_tf_env.observation_spec()
action_spec = train_tf_env.action_spec()
time_step_spec = train_tf_env.time_step_spec()

train_step = train_utils.create_train_step()

tf_agent = utils.create_agent(action_spec, observation_spec, time_step_spec, train_step, actor_fc_layer_params, critic_joint_fc_layer_params, alpha_learning_rate=alpha_learning_rate, actor_learning_rate=actor_learning_rate, critic_learning_rate=critic_learning_rate, gamma=gamma)

reverb_replay, rb_observer = utils.create_replay_buffer(tf_agent.collect_data_spec, replay_buffer_capacity)

collect_policy = utils.create_policy(tf_agent.collect_policy)
eval_policy = utils.create_policy(tf_agent.policy)
random_policy = RandomPyPolicy(time_step_spec, action_spec)

random_actor, collect_actor, eval_actor = utils.create_actors(
    train_env,
    eval_env,
    random_policy,
    collect_policy,
    eval_policy,
    train_step,
    rb_observer,
)

train_checkpointer, policy_saver = utils.create_checkpoint(tempdir, tf_agent, reverb_replay, train_step)
agent_learner = utils.create_learner(tf_agent, train_step, tempdir, reverb_replay, batch_size)

# two train loops to speed up training in the early stages
utils.train_loop(
    agent_learner,
    collect_actor,
    eval_actor,
    random_actor,
    train_tf_env,
    eval_tf_env,
    train_checkpointer,
    policy_saver,
    train_step,
    train_episodes=initial_train_episodes,
    train_steps_per_episode=initial_train_steps_per_episode,
    initial_steps=initial_train_steps_per_episode,
    eval_steps=initial_eval_steps,
    is_initial=is_initial,
)

preceding_train_steps = initial_train_episodes * initial_train_steps_per_episode

utils.train_loop(
    agent_learner,
    collect_actor,
    eval_actor,
    random_actor,
    train_tf_env,
    eval_tf_env,
    train_checkpointer,
    policy_saver,
    train_step,
    train_episodes=train_episodes,
    train_steps_per_episode=train_steps_per_episode,
    initial_steps=train_initial_steps,
    eval_steps=eval_steps,
    is_initial=False,
    preceding_train_steps=preceding_train_steps,
)
