from envs import StartEnvironment
import train.utils as utils

from tf_agents.environments.tf_py_environment import TFPyEnvironment
from tf_agents.train.utils import train_utils
from tf_agents.policies.random_py_policy import RandomPyPolicy
import tf_agents

SIM_IP = "192.168.8.195"

num_episodes = 10000
num_steps_per_episode = 2000

replay_buffer_capacity = 100000

batch_size = 1024

critic_learning_rate = 3e-4
actor_learning_rate = 3e-4
alpha_learning_rate = 3e-4
gamma = 0.9

actor_fc_layer_params = (16,16)
critic_joint_fc_layer_params = actor_fc_layer_params

tempdir = "out/"
is_initial = utils.is_initial(tempdir)

utils.set_gpu()

train_env = StartEnvironment(ip=SIM_IP, desired_height=150, dynamic_start=True)
eval_env = StartEnvironment(ip=SIM_IP, desired_height=200, dynamic_start=False)
train_tf_env = TFPyEnvironment(train_env)
eval_tf_env = TFPyEnvironment(eval_env)

observation_spec = train_tf_env.observation_spec()
action_spec = train_tf_env.action_spec()
time_step_spec = train_tf_env.time_step_spec()

train_step = train_utils.create_train_step()

tf_agent = utils.create_agent(
    action_spec,
    observation_spec,
    time_step_spec,
    train_step,
    actor_fc_layer_params,
    critic_joint_fc_layer_params,
    alpha_learning_rate=alpha_learning_rate,
    actor_learning_rate=actor_learning_rate,
    critic_learning_rate=critic_learning_rate,
    gamma=gamma,
)

reverb_replay, rb_observer = utils.create_replay_buffer(tf_agent.collect_data_spec, replay_buffer_capacity)

collect_policy = utils.create_policy(tf_agent.collect_policy)
eval_policy = utils.create_policy(tf_agent.policy)
random_policy = RandomPyPolicy(time_step_spec, action_spec)

random_actor, collect_actor, eval_actor = utils.create_actors(
    train_env,
    eval_env,
    random_policy,
    collect_policy,
    eval_policy,
    train_step,
    rb_observer,
)

train_checkpointer, policy_saver = utils.create_checkpoint(tempdir, tf_agent, reverb_replay, train_step)
agent_learner = utils.create_learner(tf_agent, train_step, tempdir, reverb_replay, batch_size)

utils.train_loop(
    agent_learner,
    collect_actor,
    eval_actor,
    random_actor,
    train_tf_env,
    eval_tf_env,
    train_checkpointer,
    policy_saver,
    train_step,
    num_episodes,
    num_steps_per_episode,
    is_initial=is_initial,
)
